/*
 *    Copyright (c) 2018-2025, lengleng All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the yifu4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: lengleng (wangiegie@gmail.com)
 */
package com.yifu.cloud.plus.v1.yifu.social.vo;

import com.alibaba.excel.annotation.ExcelProperty;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.yifu.cloud.plus.v1.yifu.common.core.vo.RowIndex;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import com.yifu.cloud.plus.v1.yifu.common.core.constant.ExcelAttribute;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotBlank;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/**
 * 缴费库
 * @author huyc
 * @date 2022-07-22 17:01:22
 */
@Data
public class TPaymentInfoVo extends RowIndex implements Serializable {

	/**
	 * 主键
	 */
	@TableId(type = IdType.ASSIGN_ID)
	@NotBlank(message = "主键 不能为空")
	@Length(max = 32, message = "主键 不能超过32 个字符")
	@ExcelAttribute(name = "主键", isNotEmpty = true, errorInfo = "主键 不能为空", maxLength = 32)
	@Schema(description = "主键")
	@ExcelProperty("主键")
	private String id;
	/**
	 * 员工姓名
	 */
	@Length(max = 20, message = "员工姓名 不能超过20 个字符")
	@ExcelAttribute(name = "员工姓名", maxLength = 20)
	@Schema(description = "员工姓名")
	@ExcelProperty("员工姓名")
	private String empName;
	/**
	 * 员工编码
	 */
	@Length(max = 32, message = "员工编码 不能超过32 个字符")
	@ExcelAttribute(name = "员工编码", maxLength = 32)
	@Schema(description = "员工编码")
	@ExcelProperty("员工编码")
	private String empNo;
	/**
	 * 员工ID
	 */
	@NotBlank(message = "员工ID 不能为空")
	@Length(max = 32, message = "员工ID 不能超过32 个字符")
	@ExcelAttribute(name = "员工ID", isNotEmpty = true, errorInfo = "员工ID 不能为空", maxLength = 32)
	@Schema(description = "员工ID")
	@ExcelProperty("员工ID")
	private String empId;
	/**
	 * 身份证号
	 */
	@Length(max = 20, message = "身份证号 不能超过20 个字符")
	@ExcelAttribute(name = "身份证号", maxLength = 20)
	@Schema(description = "身份证号")
	@ExcelProperty("身份证号")
	private String empIdcard;
	/**
	 * 单位名称
	 */
	@Length(max = 50, message = "单位名称 不能超过50 个字符")
	@ExcelAttribute(name = "单位名称", maxLength = 50)
	@Schema(description = "单位名称")
	@ExcelProperty("单位名称")
	private String unitId;
	/**
	 * 项目名称
	 */
	@Length(max = 50, message = "项目名称 不能超过50 个字符")
	@ExcelAttribute(name = "项目名称", maxLength = 50)
	@Schema(description = "项目名称")
	@ExcelProperty("项目名称")
	private String settleDomainId;
	/**
	 * 社保户
	 */
	@Length(max = 32, message = "社保户 不能超过32 个字符")
	@ExcelAttribute(name = "社保户", maxLength = 32)
	@Schema(description = "社保户")
	@ExcelProperty("社保户")
	private String socialHousehold;
	/**
	 * 社保编号
	 */
	@Length(max = 20, message = "社保编号 不能超过20 个字符")
	@ExcelAttribute(name = "社保编号", maxLength = 20)
	@Schema(description = "社保编号")
	@ExcelProperty("社保编号")
	private String socialSecurityNo;
	/**
	 * 社保缴纳地
	 */
	@Length(max = 50, message = "社保缴纳地 不能超过50 个字符")
	@ExcelAttribute(name = "社保缴纳地", maxLength = 50)
	@Schema(description = "社保缴纳地")
	@ExcelProperty("社保缴纳地")
	private String socialPayAddr;
	/**
	 * 社保缴纳月份
	 */
	@Length(max = 6, message = "社保缴纳月份 不能超过6 个字符")
	@ExcelAttribute(name = "社保缴纳月份", maxLength = 6)
	@Schema(description = "社保缴纳月份")
	@ExcelProperty("社保缴纳月份")
	private String socialPayMonth;
	/**
	 * 社保生成月份
	 */
	@Length(max = 6, message = "社保生成月份 不能超过6 个字符")
	@ExcelAttribute(name = "社保生成月份", maxLength = 6)
	@Schema(description = "社保生成月份")
	@ExcelProperty("社保生成月份")
	private String socialCreateMonth;
	/**
	 * 锁定状态 0 未锁定 1 锁定
	 */
	@Length(max = 1, message = "锁定状态 0 未锁定 1 锁定 不能超过1 个字符")
	@ExcelAttribute(name = "锁定状态 0 未锁定 1 锁定", maxLength = 1)
	@Schema(description = "锁定状态 0 未锁定 1 锁定")
	@ExcelProperty("锁定状态 0 未锁定 1 锁定")
	private String lockStatus;
	/**
	 * 总合计
	 */
	@ExcelAttribute(name = "总合计")
	@Schema(description = "总合计")
	@ExcelProperty("总合计")
	private BigDecimal sumAll;
	/**
	 * 公积金缴纳月份
	 */
	@Length(max = 6, message = "公积金缴纳月份 不能超过6 个字符")
	@ExcelAttribute(name = "公积金缴纳月份", maxLength = 6)
	@Schema(description = "公积金缴纳月份")
	@ExcelProperty("公积金缴纳月份")
	private String providentPayMonth;
	/**
	 * 公积金生成月份
	 */
	@Length(max = 6, message = "公积金生成月份 不能超过6 个字符")
	@ExcelAttribute(name = "公积金生成月份", maxLength = 6)
	@Schema(description = "公积金生成月份")
	@ExcelProperty("公积金生成月份")
	private String providentCreateMonth;
	/**
	 * 公积金户
	 */
	@Length(max = 32, message = "公积金户 不能超过32 个字符")
	@ExcelAttribute(name = "公积金户", maxLength = 32)
	@Schema(description = "公积金户")
	@ExcelProperty("公积金户")
	private String providentHousehold;
	/**
	 * 公积金缴纳地
	 */
	@Length(max = 50, message = "公积金缴纳地 不能超过50 个字符")
	@ExcelAttribute(name = "公积金缴纳地", maxLength = 50)
	@Schema(description = "公积金缴纳地")
	@ExcelProperty("公积金缴纳地")
	private String providentPayAddr;
	/**
	 * 公积金缴纳地-省
	 */
	@Length(max = 32, message = "公积金缴纳地-省 不能超过32 个字符")
	@ExcelAttribute(name = "公积金缴纳地-省", maxLength = 32)
	@Schema(description = "公积金缴纳地-省")
	@ExcelProperty("公积金缴纳地-省")
	private String fundProvince;
	/**
	 * 公积金缴纳地-市
	 */
	@Length(max = 32, message = "公积金缴纳地-市 不能超过32 个字符")
	@ExcelAttribute(name = "公积金缴纳地-市", maxLength = 32)
	@Schema(description = "公积金缴纳地-市")
	@ExcelProperty("公积金缴纳地-市")
	private String fundCity;
	/**
	 * 公积金缴纳地-县
	 */
	@Length(max = 32, message = "公积金缴纳地-县 不能超过32 个字符")
	@ExcelAttribute(name = "公积金缴纳地-县", maxLength = 32)
	@Schema(description = "公积金缴纳地-县")
	@ExcelProperty("公积金缴纳地-县")
	private String fundTown;
	/**
	 * 社保缴纳地-省
	 */
	@Length(max = 32, message = "社保缴纳地-省 不能超过32 个字符")
	@ExcelAttribute(name = "社保缴纳地-省", maxLength = 32)
	@Schema(description = "社保缴纳地-省")
	@ExcelProperty("社保缴纳地-省")
	private String socialProvince;
	/**
	 * 社保缴纳地-市
	 */
	@Length(max = 32, message = "社保缴纳地-市 不能超过32 个字符")
	@ExcelAttribute(name = "社保缴纳地-市", maxLength = 32)
	@Schema(description = "社保缴纳地-市")
	@ExcelProperty("社保缴纳地-市")
	private String socialCity;
	/**
	 * 社保缴纳地-县
	 */
	@Length(max = 32, message = "社保缴纳地-县 不能超过32 个字符")
	@ExcelAttribute(name = "社保缴纳地-县", maxLength = 32)
	@Schema(description = "社保缴纳地-县")
	@ExcelProperty("社保缴纳地-县")
	private String socialTown;
	/**
	 * 社保ID
	 */
	@Length(max = 32, message = "社保ID 不能超过32 个字符")
	@ExcelAttribute(name = "社保ID", maxLength = 32)
	@Schema(description = "社保ID")
	@ExcelProperty("社保ID")
	private String socialId;
	/**
	 * 公积金ID
	 */
	@Length(max = 32, message = "公积金ID 不能超过32 个字符")
	@ExcelAttribute(name = "公积金ID", maxLength = 32)
	@Schema(description = "公积金ID")
	@ExcelProperty("公积金ID")
	private String fundId;
	/**
	 * 社保合计
	 */
	@ExcelAttribute(name = "社保合计")
	@Schema(description = "社保合计")
	@ExcelProperty("社保合计")
	private BigDecimal socialSum;
	/**
	 * 单位社保合计
	 */
	@ExcelAttribute(name = "单位社保合计")
	@Schema(description = "单位社保合计")
	@ExcelProperty("单位社保合计")
	private BigDecimal unitSocialSum;
	/**
	 * 个人社保合计
	 */
	@ExcelAttribute(name = "个人社保合计")
	@Schema(description = "个人社保合计")
	@ExcelProperty("个人社保合计")
	private BigDecimal socialSecurityPersonalSum;
	/**
	 * 公积金总合计
	 */
	@ExcelAttribute(name = "公积金总合计")
	@Schema(description = "公积金总合计")
	@ExcelProperty("公积金总合计")
	private BigDecimal providentSum;
	/**
	 * 社保核准表ID
	 */
	@Length(max = 32, message = "社保核准表ID 不能超过32 个字符")
	@ExcelAttribute(name = "社保核准表ID", maxLength = 32)
	@Schema(description = "社保核准表ID")
	@ExcelProperty("社保核准表ID")
	private String socialSettlementId;
	/**
	 * 公积金核准表ID
	 */
	@Length(max = 32, message = "公积金核准表ID 不能超过32 个字符")
	@ExcelAttribute(name = "公积金核准表ID", maxLength = 32)
	@Schema(description = "公积金核准表ID")
	@ExcelProperty("公积金核准表ID")
	private String fundSettlementId;
	/**
	 * 就职班组
	 */
	@Length(max = 50, message = "就职班组 不能超过50 个字符")
	@ExcelAttribute(name = "就职班组", maxLength = 50)
	@Schema(description = "就职班组")
	@ExcelProperty("就职班组")
	private String inauguralTeam;
	/**
	 * 电信编号
	 */
	@Length(max = 50, message = "电信编号 不能超过50 个字符")
	@ExcelAttribute(name = "电信编号", maxLength = 50)
	@Schema(description = "电信编号")
	@ExcelProperty("电信编号")
	private String telecomNumber;
	/**
	 * 排序字段
	 */
	@Length(max = 32, message = "排序字段 不能超过32 个字符")
	@ExcelAttribute(name = "排序字段", maxLength = 32)
	@Schema(description = "排序字段")
	@ExcelProperty("排序字段")
	private String sortTime;
	/**
	 * 代理结算id
	 */
	@Length(max = 32, message = "代理结算id 不能超过32 个字符")
	@ExcelAttribute(name = "代理结算id", maxLength = 32)
	@Schema(description = "代理结算id")
	@ExcelProperty("代理结算id")
	private String agentId;
	/**
	 * 财务账单ID
	 */
	@Length(max = 32, message = "财务账单ID 不能超过32 个字符")
	@ExcelAttribute(name = "财务账单ID", maxLength = 32)
	@Schema(description = "财务账单ID")
	@ExcelProperty("财务账单ID")
	private String financeBillId;
	/**
	 * 单位社保补缴利息
	 */
	@ExcelAttribute(name = "单位社保补缴利息")
	@Schema(description ="单位社保补缴利息")
	@ExcelProperty("单位社保补缴利息")
	private BigDecimal companyAccrual;
	/**
	 * 个人社保补缴利息
	 */
	@ExcelAttribute(name = "个人社保补缴利息")
	@Schema(description ="个人社保补缴利息")
	@ExcelProperty("个人社保补缴利息")
	private BigDecimal personalAccrual;
	/**
	 * 单位养老基数
	 */
	@ExcelAttribute(name = "单位养老基数")
	@Schema(description ="单位养老基数")
	@ExcelProperty("单位养老基数")
	private BigDecimal unitPensionSet;
	/**
	 * 单位医疗基数
	 */
	@ExcelAttribute(name = "单位医疗基数")
	@Schema(description ="单位医疗基数")
	@ExcelProperty("单位医疗基数")
	private BigDecimal unitMedicalSet;
	/**
	 * 单位失业基数
	 */
	@ExcelAttribute(name = "单位失业基数")
	@Schema(description ="单位失业基数")
	@ExcelProperty("单位失业基数")
	private BigDecimal unitUnemploymentSet;
	/**
	 * 单位工伤基数
	 */
	@ExcelAttribute(name = "单位工伤基数")
	@Schema(description ="单位工伤基数")
	@ExcelProperty("单位工伤基数")
	private BigDecimal unitInjurySet;
	/**
	 * 单位生育基数
	 */
	@ExcelAttribute(name = "单位生育基数")
	@Schema(description ="单位生育基数")
	@ExcelProperty("单位生育基数")
	private BigDecimal unitBirthSet;
	/**
	 * 个人养老基数
	 */
	@ExcelAttribute(name = "个人养老基数")
	@Schema(description ="个人养老基数")
	@ExcelProperty("个人养老基数")
	private BigDecimal personalPensionSet;
	/**
	 * 个人医疗基数
	 */
	@ExcelAttribute(name = "个人医疗基数")
	@Schema(description ="个人医疗基数")
	@ExcelProperty("个人医疗基数")
	private BigDecimal personalMedicalSet;
	/**
	 * 个人失业基数
	 */
	@ExcelAttribute(name = "个人失业基数")
	@Schema(description ="个人失业基数")
	@ExcelProperty("个人失业基数")
	private BigDecimal personalUnemploymentSet;
	/**
	 * 单位养老比例
	 */
	@ExcelAttribute(name = "单位养老比例")
	@Schema(description ="单位养老比例")
	@ExcelProperty("单位养老比例")
	private BigDecimal unitPensionPer;
	/**
	 * 单位医疗比例
	 */
	@ExcelAttribute(name = "单位医疗比例")
	@Schema(description ="单位医疗比例")
	@ExcelProperty("单位医疗比例")
	private BigDecimal unitMedicalPer;
	/**
	 * 单位失业比例
	 */
	@ExcelAttribute(name = "单位失业比例")
	@Schema(description ="单位失业比例")
	@ExcelProperty("单位失业比例")
	private BigDecimal unitUnemploymentPer;
	/**
	 * 单位工伤比例
	 */
	@ExcelAttribute(name = "单位工伤比例")
	@Schema(description ="单位工伤比例")
	@ExcelProperty("单位工伤比例")
	private BigDecimal unitInjuryPer;
	/**
	 * 单位生育比例
	 */
	@ExcelAttribute(name = "单位生育比例")
	@Schema(description ="单位生育比例")
	@ExcelProperty("单位生育比例")
	private BigDecimal unitBirthPer;
	/**
	 * 个人养老比例
	 */
	@ExcelAttribute(name = "个人养老比例")
	@Schema(description ="个人养老比例")
	@ExcelProperty("个人养老比例")
	private BigDecimal personalPensionPer;
	/**
	 * 个人医疗比例
	 */
	@ExcelAttribute(name = "个人医疗比例")
	@Schema(description ="个人医疗比例")
	@ExcelProperty("个人医疗比例")
	private BigDecimal personalMedicalPer;
	/**
	 * 个人失业比例
	 */
	@ExcelAttribute(name = "个人失业比例")
	@Schema(description ="个人失业比例")
	@ExcelProperty("个人失业比例")
	private BigDecimal personalUnemploymentPer;
	/**
	 * 单位大病比例
	 */
	@ExcelAttribute(name = "单位大病比例")
	@Schema(description ="单位大病比例")
	@ExcelProperty("单位大病比例")
	private BigDecimal unitBigailmentPer;
	/**
	 * 个人大病比例
	 */
	@ExcelAttribute(name = "个人大病比例")
	@Schema(description ="个人大病比例")
	@ExcelProperty("个人大病比例")
	private BigDecimal personalBigailmentPer;
	/**
	 * 单位养老金额
	 */
	@ExcelAttribute(name = "单位养老金额")
	@Schema(description ="单位养老金额")
	@ExcelProperty("单位养老金额")
	private BigDecimal unitPensionMoney;
	/**
	 * 单位医疗金额
	 */
	@ExcelAttribute(name = "单位医疗金额")
	@Schema(description ="单位医疗金额")
	@ExcelProperty("单位医疗金额")
	private BigDecimal unitMedicalMoney;
	/**
	 * 单位失业金额
	 */
	@ExcelAttribute(name = "单位失业金额")
	@Schema(description ="单位失业金额")
	@ExcelProperty("单位失业金额")
	private BigDecimal unitUnemploymentMoney;
	/**
	 * 单位工伤金额
	 */
	@ExcelAttribute(name = "单位工伤金额")
	@Schema(description ="单位工伤金额")
	@ExcelProperty("单位工伤金额")
	private BigDecimal unitInjuryMoney;
	/**
	 * 单位生育金额
	 */
	@ExcelAttribute(name = "单位生育金额")
	@Schema(description ="单位生育金额")
	@ExcelProperty("单位生育金额")
	private BigDecimal unitBirthMoney;
	/**
	 * 单位大病金额
	 */
	@ExcelAttribute(name = "单位大病金额")
	@Schema(description ="单位大病金额")
	@ExcelProperty("单位大病金额")
	private BigDecimal unitBigmailmentMoney;
	/**
	 * 个人养老金额
	 */
	@ExcelAttribute(name = "个人养老金额")
	@Schema(description ="个人养老金额")
	@ExcelProperty("个人养老金额")
	private BigDecimal personalPensionMoney;
	/**
	 * 个人医疗金额
	 */
	@ExcelAttribute(name = "个人医疗金额")
	@Schema(description ="个人医疗金额")
	@ExcelProperty("个人医疗金额")
	private BigDecimal personalMedicalMoney;
	/**
	 * 个人失业金额
	 */
	@ExcelAttribute(name = "个人失业金额")
	@Schema(description ="个人失业金额")
	@ExcelProperty("个人失业金额")
	private BigDecimal personalUnemploymentMoney;
	/**
	 * 个人大病金额
	 */
	@ExcelAttribute(name = "个人大病金额")
	@Schema(description ="个人大病金额")
	@ExcelProperty("个人大病金额")
	private BigDecimal personalBigmailmentMoney;
	/**
	 * 公积金编号
	 */
	@Length(max=50,message = "公积金编号 不能超过50 个字符")
	@ExcelAttribute(name = "公积金编号", maxLength = 50 )
	@Schema(description ="公积金编号")
	@ExcelProperty("公积金编号")
	private String providentNo;
	/**
	 * 单位公积金基数
	 */
	@ExcelAttribute(name = "单位公积金基数")
	@Schema(description ="单位公积金基数")
	@ExcelProperty("单位公积金基数")
	private BigDecimal unitProvidentSet;
	/**
	 * 单边公积金比例
	 */
	@ExcelAttribute(name = "单边公积金比例")
	@Schema(description ="单边公积金比例")
	@ExcelProperty("单边公积金比例")
	private BigDecimal providentPercent;
	/**
	 * 单位公积金费用
	 */
	@ExcelAttribute(name = "单位公积金费用")
	@Schema(description ="单位公积金费用")
	@ExcelProperty("单位公积金费用")
	private BigDecimal unitProvidentSum;
	/**
	 * 个人公积金基数
	 */
	@ExcelAttribute(name = "个人公积金基数")
	@Schema(description ="个人公积金基数")
	@ExcelProperty("个人公积金基数")
	private BigDecimal personalProidentSet;
	/**
	 * 个人公积金费用
	 */
	@ExcelAttribute(name = "个人公积金费用")
	@Schema(description ="个人公积金费用")
	@ExcelProperty("个人公积金费用")
	private BigDecimal personalProvidentSum;
	/**
	 * 创建者
	 */
	@Length(max = 64, message = "创建者 不能超过64 个字符")
	@ExcelAttribute(name = "创建者", maxLength = 64)
	@Schema(description = "创建者")
	@ExcelProperty("创建者")
	private String createBy;
	/**
	 * 更新人
	 */
	@Length(max = 64, message = "更新人 不能超过64 个字符")
	@ExcelAttribute(name = "更新人", maxLength = 64)
	@Schema(description = "更新人")
	@ExcelProperty("更新人")
	private String updateBy;
	/**
	 * 创建人姓名
	 */
	@Length(max = 32, message = "创建人姓名 不能超过32 个字符")
	@ExcelAttribute(name = "创建人姓名", maxLength = 32)
	@Schema(description = "创建人姓名")
	@ExcelProperty("创建人姓名")
	private String createName;
	/**
	 * 创建时间
	 */
	@ExcelAttribute(name = "创建时间", isDate = true)
	@Schema(description = "创建时间")
	@ExcelProperty("创建时间")
	private Date createTime;
	/**
	 * 更新时间
	 */
	@ExcelAttribute(name = "更新时间", isDate = true)
	@Schema(description = "更新时间")
	@ExcelProperty("更新时间")
	private Date updateTime;

}
